@@ -1,15 +1,14 @@
 """Date parsing utilities"""
 import time
 from calendar import monthrange
-from datetime import datetime
+from datetime import datetime, timezone
 from typing import Dict, Optional, Tuple
 
 import dateutil
 from dateutil.parser import ParserError, _timelex, parserinfo
-from dateutil.tz import tzutc
 
 default_sd_year = 1
-default_date = datetime(
+default_date_notz = datetime(
     year=default_sd_year,
     month=1,
     day=1,
@@ -18,8 +17,18 @@ default_date = datetime(
     second=0,
     microsecond=0,
 )
+default_date = datetime(
+    year=default_sd_year,
+    month=1,
+    day=1,
+    hour=0,
+    minute=0,
+    second=0,
+    microsecond=0,
+    tzinfo=timezone.utc,
+)
 default_ed_year = 9990
-default_enddate = datetime(
+default_enddate_notz = datetime(
     year=default_ed_year,
     month=12,
     day=31,
@@ -28,6 +37,77 @@ default_enddate = datetime(
     second=0,
     microsecond=0,
 )
+default_enddate = datetime(
+    year=default_ed_year,
+    month=12,
+    day=31,
+    hour=0,
+    minute=0,
+    second=0,
+    microsecond=0,
+    tzinfo=timezone.utc,
+)
+
+# Used https://stackoverflow.com/questions/1703546/parsing-date-time-string-with-timezone-abbreviated-name-in-python/4766400#4766400
+# with IST added
+default_timezone_info = """-12 Y
+-11 X NUT SST
+-10 W CKT HAST HST TAHT TKT
+-9 V AKST GAMT GIT HADT HNY
+-8 U AKDT CIST HAY HNP PST PT
+-7 T HAP HNR MST PDT
+-6 S CST EAST GALT HAR HNC MDT
+-5 R CDT COT EASST ECT EST ET HAC HNE PET
+-4 Q AST BOT CLT COST EDT FKT GYT HAE HNA PYT
+-3 P ADT ART BRT CLST FKST GFT HAA PMST PYST SRT UYT WGT
+-2 O BRST FNT PMDT UYST WGST
+-1 N AZOT CVT EGT
+0 Z EGST GMT UTC WET WT
+1 A CET DFT WAT WEDT WEST
+2 B CAT CEDT CEST EET SAST WAST
+3 C EAT EEDT EEST IDT MSK
+4 D AMT AZT GET GST KUYT MSD MUT RET SAMT SCT
+5 E AMST AQTT AZST HMT MAWT MVT PKT TFT TJT TMT UZT YEKT
+6 F ALMT BIOT BTT IOT KGT NOVT OMST YEKST
+7 G CXT DAVT HOVT ICT KRAT NOVST OMSST THA WIB
+8 H ACT AWST BDT BNT CAST HKT IRKT KRAST MYT PHT SGT ULAT WITA WST
+9 I AWDT IRKST JST KST PWT TLT WDT WIT YAKT
+10 K AEST ChST PGT VLAT YAKST YAPT
+11 L AEDT LHDT MAGT NCT PONT SBT VLAST VUT
+12 M ANAST ANAT FJT GILT MAGST MHT NZST PETST PETT TVT WFT
+13 FJST NZDT
+11.5 NFT
+10.5 ACDT LHST
+9.5 ACST
+6.5 CCT MMT
+5.75 NPT
+5.5 IST SLT
+4.5 AFT IRDT
+3.5 IRST
+-2.5 HAT NDT
+-3.5 HNT NST NT
+-4.5 HLV VET
+-9.5 MART MIT"""
+
+
+def get_tzinfos(timezone_info: str) -> Dict[str, int]:
+    """Get tzinfos dictionary used by dateutil from timezone information string
+
+    Args:
+        timezone_info (str): Timezones information string
+
+    Returns:
+        Dict[str, int]: tzinfos dictionary
+    """
+    tzinfos = {}
+    for tz_descr in map(str.split, timezone_info.split("\n")):
+        tz_offset = int(float(tz_descr[0]) * 3600)
+        for tz_code in tz_descr[1:]:
+            tzinfos[tz_code] = tz_offset
+    return tzinfos
+
+
+default_tzinfos = get_tzinfos(default_timezone_info)
 
 
 # Ugly copy and paste from dateutil.parser._parser._ymd with dayfirst modified to mean day is outer value
@@ -568,31 +648,83 @@ def parse(
 def parse_date_range(
     string: str,
     date_format: Optional[str] = None,
+    timezone_handling: int = 0,
     fuzzy: Optional[Dict] = None,
+    include_microseconds: bool = False,
     zero_time: bool = False,
+    max_starttime: bool = False,
+    max_endtime: bool = False,
+    default_timezones: Optional[str] = None,
 ) -> Tuple[datetime, datetime]:
-    """Parse date from string using specified format (setting any time elements to 0 if zero_time is True).
-    If no format is supplied, the function will guess. For unambiguous formats, this should be fine.
-    Returns date range in dictionary keys startdate and enddate. If a dictionary is supplied in the fuzzy parameter,
-    then dateutil's fuzzy parsing is used and the results returned in the dictionary in keys startdate, enddate,
-    date (the string elements used to make the date) and nondate (the non date part of the string).
+    """Parse date from string using specified date_format if given and return datetime
+    date range in dictionary keys startdate and enddate. If no date_format is supplied,
+    the function will guess, which for unambiguous formats, should work fine.
+
+    By default, no timezone information will be parsed and the returned datetime will
+    have timezone UTC. To change this behaviour, timezone_handling should be changed
+    from its default of 0. If it is 1, then no timezone information will be parsed and a
+    naive datetime will be returned. If it is 2 or more, then timezone information will
+    be parsed. For 2, failure to parse timezone will result in a naive datetime. For 3,
+    failure to parse timezone will result in the timezone being set to UTC. For 4 and 5,
+    the time will be converted from whatever timezone is identified to UTC. For 4,
+    failure to parse timezone will result in a naive (local) datetime converted to UTC.
+    For 5, failure to parse timezone will result in the timezone being set to UTC.
+
+    To parse a date within a string containing other text, you can supply a dictionary
+    in the fuzzy parameter. In this case, dateutil's fuzzy parsing is used and the
+    results returned in the dictionary in keys startdate, enddate, date (the string
+    elements used to make the date) and nondate (the non date part of the string).
+
+    By default, microseconds are ignored (set to 0), but can be included by setting
+    include_microseconds to True. Any time elements are set to 0 if zero_time is True.
+    If max_starttime is True, then the start date's time is set to 23:59:59. If
+    max_endtime is True, then the end date's time is set to 23:59:59.
+
+    When inferring time zones, a default set of time zones will be used unless
+    overridden by passing in default_timezones which is a string of the form:
+
+        -11 X NUT SST
+        -10 W CKT HAST HST TAHT TKT
 
     Args:
         string (str): Dataset date string
         date_format (Optional[str]): Date format. If None is given, will attempt to guess. Defaults to None.
+        timezone_handling (int): Timezone handling. See description. Defaults to 0 (ignore timezone, return UTC).
         fuzzy (Optional[Dict]): If dict supplied, fuzzy matching will be used and results returned in dict
+        include_microseconds (bool): Includes microseconds if True. Defaults to False.
         zero_time (bool): Zero time elements of datetime if True. Defaults to False.
+        max_starttime (bool): Make start date time component 23:59:59:999999. Defaults to False.
+        max_endtime (bool): Make end date time component 23:59:59:999999. Defaults to False.
+        default_timezones (Optional[str]): Timezone information. Defaults to None. (Internal default).
 
     Returns:
         Tuple[datetime,datetime]: Tuple containing start date and end date
     """
     if date_format is None or fuzzy is not None:
+        if timezone_handling >= 2:
+            if default_timezones is None:
+                tzinfos = default_tzinfos
+            else:
+                tzinfos = get_tzinfos(default_timezones)
+            ignoretz = False
+        else:
+            ignoretz = True
+            tzinfos = None
+
         if fuzzy is not None:
             parsed_string1 = parse(
-                string, fuzzy_with_tokens=True, default=default_date
+                string,
+                fuzzy_with_tokens=True,
+                default=default_date_notz,
+                ignoretz=ignoretz,
+                tzinfos=tzinfos,
             )
             parsed_string2 = parse(
-                string, fuzzy_with_tokens=True, default=default_enddate
+                string,
+                fuzzy_with_tokens=True,
+                default=default_enddate_notz,
+                ignoretz=ignoretz,
+                tzinfos=tzinfos,
             )
             startdate = parsed_string1[0]
             enddate = parsed_string2[0]
@@ -603,8 +735,18 @@ def parse_date_range(
                 fuzzy["nondate"] = None
             fuzzy["date"] = parsed_string1[2]
         else:
-            startdate = parse(string, default=default_date)
-            enddate = parse(string, default=default_enddate)
+            startdate = parse(
+                string,
+                default=default_date_notz,
+                ignoretz=ignoretz,
+                tzinfos=tzinfos,
+            )
+            enddate = parse(
+                string,
+                default=default_enddate_notz,
+                ignoretz=ignoretz,
+                tzinfos=tzinfos,
+            )
         if (
             startdate.year == default_sd_year
             and enddate.year == default_ed_year
@@ -638,10 +780,45 @@ def parse_date_range(
             startdate = startdate.replace(month=default_date.month)
             enddate = enddate.replace(month=default_enddate.month)
     if zero_time:
+        if not max_starttime:
+            startdate = startdate.replace(
+                hour=0, minute=0, second=0, microsecond=0
+            )
+        if not max_endtime:
+            enddate = enddate.replace(
+                hour=0, minute=0, second=0, microsecond=0
+            )
+    if max_starttime:
         startdate = startdate.replace(
-            hour=0, minute=0, second=0, microsecond=0
+            hour=23, minute=59, second=59, microsecond=999999
         )
-        enddate = enddate.replace(hour=0, minute=0, second=0, microsecond=0)
+    if max_endtime:
+        enddate = enddate.replace(
+            hour=23, minute=59, second=59, microsecond=999999
+        )
+    if timezone_handling == 0:
+        startdate = startdate.replace(tzinfo=timezone.utc)
+        enddate = enddate.replace(tzinfo=timezone.utc)
+    elif timezone_handling == 3:
+        if startdate.tzinfo is None:
+            startdate = startdate.replace(tzinfo=timezone.utc)
+        if enddate.tzinfo is None:
+            enddate = enddate.replace(tzinfo=timezone.utc)
+    elif timezone_handling == 4:
+        startdate = startdate.astimezone(timezone.utc)
+        enddate = enddate.astimezone(timezone.utc)
+    elif timezone_handling == 5:
+        if startdate.tzinfo is None:
+            startdate = startdate.replace(tzinfo=timezone.utc)
+        else:
+            startdate = startdate.astimezone(timezone.utc)
+        if enddate.tzinfo is None:
+            enddate = enddate.replace(tzinfo=timezone.utc)
+        else:
+            enddate = enddate.astimezone(timezone.utc)
+    if not include_microseconds:
+        startdate = startdate.replace(microsecond=0)
+        enddate = enddate.replace(microsecond=0)
     if fuzzy is not None:
         fuzzy["startdate"] = startdate
         fuzzy["enddate"] = enddate
@@ -651,27 +828,74 @@ def parse_date_range(
 def parse_date(
     string: str,
     date_format: Optional[str] = None,
+    timezone_handling: int = 0,
     fuzzy: Optional[Dict] = None,
+    include_microseconds: bool = False,
     zero_time: bool = False,
+    max_time: bool = False,
+    default_timezones: Optional[str] = None,
 ) -> datetime:
-    """Parse date from string using specified format (setting any time elements to 0 if zero_time is True).
-    If no format is supplied, the function will guess. For unambiguous formats, this should be fine.
-    Returns a datetime object. Raises exception for dates that are missing year, month or day.
-    If a dictionary is supplied in the fuzzy parameter, then dateutil's fuzzy parsing is used and the results
-    returned in the dictionary in keys startdate, enddate, date (the string elements used to make the date) and
-    nondate (the non date part of the string).
+    """Parse date from string using specified date_format and return a datetime object.
+    Raises exception for dates that are missing year, month or day. If no date_format is
+    supplied, the function will guess, which for unambiguous formats, should work fine.
+
+    By default, no timezone information will be parsed and the returned datetime will
+    have timezone UTC. To change this behaviour, timezone_handling should be changed
+    from its default of 0. If it is 1, then no timezone information will be parsed and a
+    naive datetime will be returned. If it is 2 or more, then timezone information will
+    be parsed. For 2, failure to parse timezone will result in a naive datetime. For 3,
+    failure to parse timezone will result in the timezone being set to UTC. For 4 and 5,
+    the time will be converted from whatever timezone is identified to UTC. For 4,
+    failure to parse timezone will result in a naive (local) datetime converted to UTC.
+    For 5, failure to parse timezone will result in the timezone being set to UTC.
+
+    To parse a date within a string containing other text, you can supply a dictionary
+    in the fuzzy parameter. In this case, dateutil's fuzzy parsing is used and the
+    results returned in the dictionary in keys startdate, enddate, date (the string
+    elements used to make the date) and nondate (the non date part of the string).
+
+    By default, microseconds are ignored (set to 0), but can be included by setting
+    include_microseconds to True. Any time elements are set to 0 if zero_time is True.
+    If max_starttime is True, then the start date's time is set to 23:59:59. If
+    max_endtime is True, then the end date's time is set to 23:59:59.
+
+    When inferring time zones, a default set of time zones will be used unless
+    overridden by passing in default_timezones which is a string of the form:
+
+        -11 X NUT SST
+        -10 W CKT HAST HST TAHT TKT
 
     Args:
         string (str): Dataset date string
         date_format (Optional[str]): Date format. If None is given, will attempt to guess. Defaults to None.
+        timezone_handling (int): Timezone handling. See description. Defaults to 0 (ignore timezone, return UTC).
         fuzzy (Optional[Dict]): If dict supplied, fuzzy matching will be used and results returned in dict
+        include_microseconds (bool): Includes microseconds if True. Defaults to False.
         zero_time (bool): Zero time elements of datetime if True. Defaults to False.
+        max_time (bool): Make date time component 23:59:59:999999. Defaults to False.
+        default_timezones (Optional[str]): Timezone information. Defaults to None. (Internal default).
 
     Returns:
         datetime: The parsed date
     """
+    if max_time:
+        max_starttime = True
+        max_endtime = True
+        zero_time = False
+    else:
+        max_starttime = False
+        max_endtime = False
+
     startdate, enddate = parse_date_range(
-        string, date_format=date_format, fuzzy=fuzzy, zero_time=zero_time
+        string,
+        date_format=date_format,
+        timezone_handling=timezone_handling,
+        fuzzy=fuzzy,
+        include_microseconds=include_microseconds,
+        zero_time=zero_time,
+        max_starttime=max_starttime,
+        max_endtime=max_endtime,
+        default_timezones=default_timezones,
     )
     if startdate != enddate:
         raise ParserError("date is not a specific day!")
@@ -705,20 +929,22 @@ def get_timestamp_from_datetime(date: datetime) -> float:
             + date.microsecond / 1e6
         )
     else:
-        return (date - datetime(1970, 1, 1, tzinfo=tzutc())).total_seconds()
+        return (
+            date - datetime(1970, 1, 1, tzinfo=timezone.utc)
+        ).total_seconds()
 
 
 def get_datetime_from_timestamp(
     timestamp: float,
-    timezone: datetime.tzinfo = tzutc,
-    today: datetime = datetime.now(tzutc()),
+    timezone: datetime.tzinfo = timezone.utc,
+    today: datetime = datetime.utcnow(),
 ) -> datetime:
     """Convert timestamp to datetime.
 
     Args:
         timestamp (float): Timestamp to convert
         timezone (datetime.tzinfo): Timezone to use
-        today (datetime): Today's date. Defaults to datetime.now().
+        today (datetime): Today's date. Defaults to datetime.utcnow.
 
     Returns:
         datetime: Date of timestamp
